# AUTOGENERATED! DO NOT EDIT! File to edit: 04_config.ipynb (unless otherwise specified).

__all__ = ['wiki_site', 'wiki_options', 'wiki_normal', 'regex_wortart', 'regex_discard', 'regex_word_type',
           'regex_proper_spelling', 'regex_basic_word', 'complex_kinds']

# Cell
wiki_site = 'https://de.wiktionary.org'
wiki_options = '/w/api.php?action=parse&format=json&prop=wikitext&disabletoc=&page='
wiki_normal = '/wiki/'
regex_wortart = '^===[^=]+===[^=]'
regex_discard = '^====[ ]*.*===='
regex_word_type = 'Wortart[ ]*\|[ ]*([\w ]+)[ ]*\|[ ]*Deutsch'
regex_proper_spelling = '{{.*\|(.*)\|.*}}'
regex_basic_word = '{{Grundformverweis.*\|([\w ]+)}}'

# Cell
complex_kinds = [
    'Konjugierte Form', 'Deklinierte Form', 'Komparativ', 'Konjugierte Form',
    'Partizip I', 'Partizip II'
]

# Cell
_all_kinds = [
    'Abkürzung', 'Adjektiv', 'Adverb', 'Antwortpartikel', 'Artikel',
    'Deklinierte Form', 'Demonstrativpronomen', 'Eigenname', 'Fokuspartikel',
    'Gradpartikel', 'Hilfsverb', 'Indefinitpronomen', 'Interjektion',
    'Interrogativpronomen', 'Komparativ', 'Konjugierte Form', 'Konjunktion',
    'Konjunktionaladverb', 'Kontraktion', 'Lokaladverb', 'Modaladverb',
    'Modalpartikel', 'Nachname', 'Negationspartikel', 'Numerale', 'Partikel',
    'Partizip I', 'Partizip II', 'Personalpronomen', 'Possessivpronomen',
    'Postposition', 'Pronomen', 'Pronominaladverb', 'Präposition',
    'Reflexivpronomen', 'Relativpronomen', 'Subjunktion', 'Substantiv',
    'Temporaladverb', 'Toponym', 'Verb', 'Vergleichspartikel', 'Vorname',
    'Zahlklassifikator'
]

# Cell
_line_width = 55